from django.contrib.auth.models import Permission
from wagtail import hooks

from .views import WaffleViewSetGroup
from .bulk_actions import (
    EnableForAllBulkAction,
    DisableForAllBulkAction,
    EnableSwitchesBulkAction,
    DisableSwitchesBulkAction,
    DeleteFlagBulkAction,
    DeleteSwitchBulkAction,
    DeleteSampleBulkAction
)

@hooks.register("register_admin_viewset")
def register_viewset():
    return WaffleViewSetGroup()

@hooks.register("register_permissions")
def permissions_hook():
    return Permission.objects.filter(
        content_type__app_label="waffle",
            )

for action_class in [EnableForAllBulkAction, DisableForAllBulkAction, EnableSwitchesBulkAction, DisableSwitchesBulkAction, DeleteFlagBulkAction, DeleteSwitchBulkAction, DeleteSampleBulkAction]:
    hooks.register("register_bulk_action", action_class)


from django.utils.translation import gettext_lazy as _
from django.utils.functional import classproperty
from wagtail.admin.views.bulk_action import BulkAction
from wagtail.snippets.permissions import get_permission_name
from wagtail.admin.admin_url_finder import AdminURLFinder
from waffle.models import Flag, Sample, Switch

class WaffleBulkAction(BulkAction):
    template_name = "waffle/action.html"
    
    def check_perm(self, obj):
        self.can_change_items = self.request.user.has_perm(
            get_permission_name("change", self.model)
        )
        return self.can_change_items
    
    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context.update(
            {
                "waffle_type": (
                    self.model._meta
                ),
                "action": (
                    self.action_type
                ),
            }
        )
        return context
    

class EnableForAllBulkAction(WaffleBulkAction):
    models = [Flag]
    aria_label = _("Enable selected flags for everyone")
    display_name = _("Enable for all")
    action_type = "enable"
    
    @classmethod
    def execute_action(cls, objects, **kwargs):
        cls.get_default_model().objects.filter(
            pk__in=[obj.pk for obj in objects]
        ).update(everyone=True)
        return len(objects), 0


class DisableForAllBulkAction(WaffleBulkAction):
    models = [Flag]
    aria_label = _("Disable selected flags for everyone")
    display_name = _("Disable for all")
    action_type = "disable"
    
    @classmethod
    def execute_action(cls, objects, **kwargs):
        cls.get_default_model().objects.filter(
            pk__in=[obj.pk for obj in objects]
        ).update(everyone=False)
        return len(objects), 0
    

class EnableSwitchesBulkAction(WaffleBulkAction):
    models = [Switch]
    aria_label = _("Enable selected switches")
    display_name = _("Enable for all")
    action_type = "enable"
    
    @classmethod
    def execute_action(cls, objects, **kwargs):
        cls.get_default_model().objects.filter(
            pk__in=[obj.pk for obj in objects]
        ).update(active=True)
        return len(objects), 0


class DisableSwitchesBulkAction(WaffleBulkAction):
    models = [Switch]
    aria_label = _("Disable selected switches")
    display_name = _("Disable for all")
    action_type = "disable"
    
    @classmethod
    def execute_action(cls, objects, **kwargs):
        cls.get_default_model().objects.filter(
            pk__in=[obj.pk for obj in objects]
        ).update(active=False)
        return len(objects), 0
    

class DeleteWaffleBulkAction(WaffleBulkAction):
    aria_label = _("Delete selected")
    action_type = "delete"
    display_name = _("Delete")
    classes = {"serious"}

    def check_perm(self, object):
        self.can_delete_items = self.request.user.has_perm(
            get_permission_name("delete", self.model)
        )
        return self.can_delete_items
    
    @classmethod
    def execute_action(cls, objects, **kwargs):
        cls.get_default_model().objects.filter(
            pk__in=[obj.pk for obj in objects]
        ).delete()
        return len(objects), 0


class DeleteFlagBulkAction(DeleteWaffleBulkAction):
    models = [Flag]


class DeleteSwitchBulkAction(DeleteWaffleBulkAction):
    models = [Switch]

    
class DeleteSampleBulkAction(DeleteWaffleBulkAction):
    models = [Sample]



from wagtail.admin.viewsets.model import ModelViewSet
from wagtail.admin.viewsets.base import ViewSetGroup
from wagtail.admin.views.generic import IndexView
from wagtail.admin.ui.tables import (
    BulkActionsCheckboxColumn,
    InlineActionsTable,
)
from waffle.models import Flag, Sample, Switch


class IndexView(IndexView):
    table_class = InlineActionsTable
    template_name = "waffle/index.html"

    def get_columns(self):
        return [
            BulkActionsCheckboxColumn("checkbox", accessor=lambda obj: obj),
            *super().get_columns(),
        ]

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)

        context.update(
            {
                "model_opts": self.model._meta,
            }
        )

        return context
    
class BaseAdmin(ModelViewSet):
    search_fields = ('name', 'note')
    exclude_form_fields = ["created", "modified"]
    index_view_class = IndexView


class FlagAdmin(BaseAdmin):
    model = Flag
    list_display = ('name', 'note', 'everyone', 'percent', 'superusers',
                    'staff', 'authenticated', 'languages')
    list_filter = ('everyone', 'superusers', 'staff', 'authenticated')
    ordering = ('-id',)

class SampleAdmin(BaseAdmin):
    model = Sample
    list_display = ('name', 'percent', 'note', 'created', 'modified')
    ordering = ('-id',)

class SwitchAdmin(BaseAdmin):
    model = Switch
    list_display = ('name', 'active', 'note', 'created', 'modified')
    list_filter = ('active',)
    ordering = ('-id',)

class WaffleViewSetGroup(ViewSetGroup):
    menu_label = "Features"
    menu_icon = "pick"
    add_to_admin_menu = True
    items = (SwitchAdmin(), FlagAdmin(), SampleAdmin())